package ru.ldeloff.servermonitorbot.controller;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.telegram.telegrambots.bots.TelegramLongPollingBot;
import org.telegram.telegrambots.meta.api.objects.Update;
import ru.ldeloff.servermonitorbot.model.TelegramBot;
import ru.ldeloff.servermonitorbot.repository.SshRepository;
import ru.ldeloff.servermonitorbot.service.command.FirstUseCommand;
import ru.ldeloff.servermonitorbot.service.command.GetStatusSessions;
import ru.ldeloff.servermonitorbot.service.command.SwitchToMainMenu;
import ru.ldeloff.servermonitorbot.service.command.cputemp.CpuTempAggregateCommand;
import ru.ldeloff.servermonitorbot.service.command.cputemp.CpuTempCommand;
import ru.ldeloff.servermonitorbot.service.command.hddtemp.HddTempAggregateCommand;
import ru.ldeloff.servermonitorbot.service.command.hddtemp.HddTempCommand;
import ru.ldeloff.servermonitorbot.service.command.uname.UnameAggregateCommand;
import ru.ldeloff.servermonitorbot.service.command.uname.UnameCommand;

@Service
@Slf4j
@RequiredArgsConstructor
public class TelegramBotController extends TelegramLongPollingBot {
    final TelegramBot telegramBot;
    final SshRepository sshRepository;

    final FirstUseCommand firstUseCommand;
    final GetStatusSessions getStatusSessions;
    final UnameAggregateCommand unameAggregateCommand;
    final SwitchToMainMenu switchToMainMenu;
    final UnameCommand unameCommand;
    final CpuTempAggregateCommand cpuTempAggregateCommand;
    final CpuTempCommand cpuTempCommand;
    final HddTempAggregateCommand hddTempAggregateCommand;
    final HddTempCommand hddTempCommand;

    @Override
    public void onUpdateReceived(Update update) {
        if (update.hasMessage()) {
            if (update.getMessage().hasText()) {
                String messageText = update.getMessage().getText();
                switch (messageText) {
                    case "/start" -> {
                        firstUseCommand.execute(update, this);
                    }
                    case "Статус" -> {
                        getStatusSessions.execute(update, this);
                    }
                    case "uname" -> {
                        unameAggregateCommand.execute(update, this);
                    }
                    case "CPU.temp" -> {
                        cpuTempAggregateCommand.execute(update, this);
                    }
                    case "HDD.temp" -> {
                        hddTempAggregateCommand.execute(update, this);
                    }
                    default -> {
                        switchToMainMenu.execute(update, this);
                    }
                }
            }
        } else if (update.hasCallbackQuery()) {
            String [] tags = update.getCallbackQuery().getData().split(":");
            if (tags.length > 1) {
                switch (tags[0]) {
                    case "uname" -> {
                        unameCommand.execute(update, this);
                    }
                    case "CPU.temp" -> {
                        cpuTempCommand.execute(update, this);
                    }
                    case "HDD.temp" -> {
                        hddTempCommand.execute(update, this);
                    }
                    default -> {
                        switchToMainMenu.execute(update, this);
                    }
                }
            } else {
                switchToMainMenu.execute(update, this);
            }
        }
    }
    @Override
    public String getBotUsername() {
        return telegramBot.getBotUsername();
    }
    @Override
    public String getBotToken() {
        return telegramBot.getBotToken();
    }
    @Override
    public void onClosing() {
        super.onClosing();
        sshRepository.disconnectSessions();
    }
}
